chiark / gitweb /
volume_id: use udev-provided log-level
[elogind.git] / udev.c
diff --git a/udev.c b/udev.c
index 6d1d679d9a0ccd446f988cb605c407dd545c123f..3f76e98b2413e996ebcef1b02ad723e7d1a11b48 100644 (file)
--- a/udev.c
+++ b/udev.c
@@ -54,8 +54,10 @@ void log_message(int priority, const char *format, ...)
 }
 #endif
 
/* decide if we should manage the whole hotplug event
* for now look if the kernel calls udevsend instead of /sbin/hotplug
  */
 static int manage_hotplug_event(void) {
        char helper[256];
@@ -70,13 +72,15 @@ static int manage_hotplug_event(void) {
        if (fd < 0)
                return 0;
 
-       len = read(fd, helper, 256);
+       len = read(fd, helper, sizeof(helper)-1);
        close(fd);
 
        if (len < 0)
                return 0;
        helper[len] = '\0';
 
+       if (helper[0] == '\0' || helper[0] == '\n')
+               return 1;
        if (strstr(helper, "udevsend"))
                return 1;
 
@@ -141,14 +145,14 @@ int main(int argc, char *argv[], char *envp[])
        if (!subsystem && argc == 2)
                subsystem = argv[1];
 
-       udev_init_device(&udev, devpath, subsystem);
+       udev_init_device(&udev, devpath, subsystem, action);
 
        if (!action || !subsystem || !devpath) {
                err("action, subsystem or devpath missing");
                goto hotplug;
        }
 
-       /* export logging flag, as called scripts may want to do the same as udev */
+       /* export logging flag, as called programs may want to do the same as udev */
        if (udev_log_priority) {
                char priority[32];
 
@@ -157,6 +161,8 @@ int main(int argc, char *argv[], char *envp[])
        }
 
        if (udev.type == DEV_BLOCK || udev.type == DEV_CLASS || udev.type == DEV_NET) {
+               udev_rules_init();
+
                if (strcmp(action, "add") == 0) {
                        /* wait for sysfs and possibly add node */
                        dbg("udev add");
@@ -178,9 +184,6 @@ int main(int argc, char *argv[], char *envp[])
 
                        wait_for_class_device(class_dev, &error);
 
-                       /* init rules */
-                       udev_rules_init();
-
                        /* name, create node, store in db */
                        retval = udev_add_device(&udev, class_dev);
 
@@ -195,16 +198,27 @@ int main(int argc, char *argv[], char *envp[])
                                goto hotplug;
                        }
 
+                       udev_rules_get_run(&udev);
+                       if (udev.ignore_device) {
+                               dbg("device event will be ignored");
+                               goto hotplug;
+                       }
+
                        /* get node from db, remove db-entry, delete created node */
                        retval = udev_remove_device(&udev);
                }
 
-               /* run dev.d/ scripts if we created/deleted a node or changed a netif name */
-               if (udev.devname[0] != '\0') {
+               if (udev.devname[0] != '\0')
                        setenv("DEVNAME", udev.devname, 1);
-                       if (udev_dev_d)
-                               udev_multiplex_directory(&udev, DEVD_DIR, DEVD_SUFFIX);
+
+               if (udev_run && !list_empty(&udev.run_list)) {
+                       struct name_entry *name_loop;
+
+                       dbg("executing run list");
+                       list_for_each_entry(name_loop, &udev.run_list, node)
+                               execute_command(name_loop->name, udev.subsystem);
                }
+
        } else if (udev.type == DEV_DEVICE) {
                if (strcmp(action, "add") == 0) {
                        /* wait for sysfs */
@@ -230,10 +244,11 @@ int main(int argc, char *argv[], char *envp[])
        }
 
 hotplug:
-       udev_cleanup_device(&udev);
        if (udev_hotplug_d && managed_event)
                udev_multiplex_directory(&udev, HOTPLUGD_DIR, HOTPLUG_SUFFIX);
 
+       udev_cleanup_device(&udev);
+
        logging_close();
        return retval;
 }
@@ -54,8 +54,10 @@ void log_message(int priority, const char *format, ...)
 #endif
 
-/* decide if we should manage the whole hotplug event
- * for now look if the kernel calls udevsend instead of /sbin/hotplug
+/* Decide if we should manage the whole uevent, including multiplexing
+ * of the hotplug directories.
+ * For now look if the kernel calls udevsend instead of /sbin/hotplug,
+ * or the uevent-helper in /proc/sys/kernel/hotplug is empty.
  */
 static int manage_hotplug_event(void) {